package modelo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.UUID;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlValue;

import modelo.prototypes.PrototipoListaPropiedadLinea;

// TODO: Auto-generated Javadoc
/**
 * Clase abstracta que contiene atributos y mtodos comunes a una componente elemental y a una componente compuesta.
 */
@XmlAccessorType(XmlAccessType.NONE)
@XmlSeeAlso({ComponenteElemental.class, ComponenteCompuesta.class})
public abstract class Componente implements Cloneable {

	/** El nombre de la componente. */
	@XmlAttribute(name="Nombre")
	private String nombre;
	
	/** El nivel de anidamiento de una componente. */
	@XmlAttribute(name="Anidamiento")
	private int anidamiento;
	
	/** Es derivada la componente?. */
	@XmlAttribute(name="Derivada")
	private Boolean derivada;
	
	/** Es opcional la componente?. */
	@XmlAttribute(name="Opcional")
	private Boolean opcional;
	
	/** Estructura a la que pertenece la componente. */
	protected Estructura estructura;
	
	/** Componente contenedora de la componente actual. Null en el caso de que se trate
	 * de la componente raz */
	private ComponenteCompuesta parent;
	
	/** Valores de las propiedades asociadas a la componente. */
	@XmlElement(name="Propiedades")
	private ArrayList<PropiedadLinea> propiedades;
	
	/** Identificador */
	@XmlID
	private String id = UUID.randomUUID().toString();
	
	/** Identificador de la estructura origen, en el caso de que se trate de una componente derivada. */
	@XmlAttribute(name="IDEstructuraOrigen")
	private String idEstructuraOrigen;
	
	/** Identificador de la componente origen, en el caso de que se trate de una componente derivada. */
	@XmlAttribute(name="IDComponenteOrigen")
	private String idComponenteOrigen;
	
	/**
	 * Inicializa un objeto de tipo componente.
	 */
	public Componente() {
		propiedades = generarPropiedades();
	}
	
	
	
	/**
	 * Inicializa un objeto de tipo componente.
	 *
	 * @param nombre the nombre
	 * @param anidamiento the anidamiento
	 * @param derivada the derivada
	 * @param opcional the opcional
	 * @param estructura the estructura
	 * @param parent the parent
	 */
	public Componente(String nombre, int anidamiento, Boolean derivada,
			Boolean opcional, Estructura estructura, ComponenteCompuesta parent) {
		this.nombre = nombre;
		this.anidamiento = anidamiento;
		this.derivada = derivada;
		this.opcional = opcional;
		this.estructura = estructura;
		this.parent = parent;
		
		propiedades = generarPropiedades();
	}



	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public abstract Object clone() throws CloneNotSupportedException;



	/**
	 * Derivar una componente crea una un clon de la componente original, con la diferencia
	 * de que se genera un nuevo identificador para la componente clonada, y se mantienen referencias
	 * a la componente a la que hace referencia la componente clonada, as como a su estructura.
	 *
	 * @return the object
	 */
	public Object derivar(){
		try {
			Componente componenteClonada = (Componente) this.clone();
			generarReferenciasDerivacion(componenteClonada);
			return componenteClonada;
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
	}
	/**
	 * Desplazar hacia abajo una posicin la componente, en el caso de que sto sea posible.
	 */
	public void desplazarHaciaAbajo() {
		
		int posicion = parent.getPosicionComponente(this.getId());
		
		/* Si est en la ltima posicin, no se puede bajar hacia abajo */
		if(posicion < parent.getComponentes().size()-1){
			// Swap entre los elementos
			Collections.swap(parent.getComponentes(), posicion, posicion+1);
			
			/* Actualizar smbolos */
			parent.actualizarSimbolos();
		}
		
	}
	/**
	 * Desplazar hacia arriba una posicin la componente, en el caso de que sto sea posible.
	 */
	public void desplazarHaciaArriba(){
		
		int posicion = parent.getPosicionComponente(this.getId());
		
		/* Si est en la primera posicin, no se puede subir hacia arriba */
		if(posicion > 0){
			// Swap entre los elementos
			Collections.swap(parent.getComponentes(), posicion, posicion-1);
			
			/* Actualizar smbolos */
			parent.actualizarSimbolos();
		}
		
	}
	/**
	 * Desplazar hacia derecha (nivel de anidamiento superior) la componente.
	 *
	 * @return flag, indicando el tipo de desplazamiento que se ha hecho:
	 * -1: no se ha hecho
	 * 1: se ha insertado la compoennte en la componente superior
	 * 2: se ha insertado la componente en la componente inferior
	 * 3: se convierte la componente superior en una componente compuesta y se integra la componente actual en ella.
	 */
	public int desplazarHaciaDerecha() {
		// posicin de la componente actual en el parent
		int posicion = getParent().getPosicionComponente(getId());

		// En primer lugar se intenta integrarlo en la componente compleja superior
		if(posicion-1 >= 0){ // Existe una componente anterior
			if(getParent().getComponentes().get(posicion-1).getClass() == ComponenteCompuesta.class){ // La componente anterior es compuesta (cc2)

				// Eliminar de su posicin actual
				getParent().eliminarComponente(posicion);
				
				// Actualizar simbolos del anterior parent
				getParent().actualizarSimbolos();

				// Cambio de parent
				setParent((ComponenteCompuesta) getParent().getComponentes().get(posicion-1));

				// Cambio de nivel de anidamiento
				setAnidamiento(getAnidamiento()+1);

				// Se integra la componente componente actual (cc1) en cc2 en la ltima posicin
				getParent().getComponentes().add(getParent().getComponentes().size(), this);
				
				// Actualizar smbolos el parent actual
				getParent().actualizarSimbolos();

				return 1;

			}
		}
		// En segundo lugar se intenta integrarla en la componente compleja inferior
		if(posicion+1 < getParent().getComponentes().size()){
			if(getParent().getComponentes().get(posicion+1).getClass() == ComponenteCompuesta.class){
				// Eliminar de su posicin actual
				getParent().eliminarComponente(posicion);
				
				// Actualizar simbolos del anterior parent
				getParent().actualizarSimbolos();

				// Cambio de parent
				setParent((ComponenteCompuesta) getParent().getComponentes().get(posicion)); // no es posicion+1, porque despues del borrado posicion+1 pasa a estar en posicion

				// Cambio de nivel de anidamiento
				setAnidamiento(getAnidamiento()+1);

				// Se integra la componente componente actual (cc1) en cc2 en la ltima posicin
				getParent().getComponentes().add(this);
				
				// Actualizar simbolos del parent actual
				getParent().actualizarSimbolos();

				return 2;
			}
		}
		
		/*
		 * POR LTIMO, SI NO SE HA HECHO NINGUNA DE LAS ANTERIORES SE TRANSFORMA LA COMPONENTE SUPERIOR EN UNA
		 * COMPONENTE COMPUESTA, SE ELIMINA LA COMPONENTE DE SU SITIO ACTUAL Y SE INSERTA EN LA COMPONENTE SUPERIOR.
		 */
		
		if(posicion-1 >= 0){ // existe componente anterior
			
			// Obtener componente superior
			Componente ce = this.getParent().getComponentes().get(posicion-1);
			
			// Modificar su clase y convertirla en compuesta
			ce.modificarClase();
			
			// Obtener la componente compuesta
			ComponenteCompuesta cc = (ComponenteCompuesta) this.getParent().getComponentes().get(posicion-1);
			
			// Eliminar de la posicin actual
			
			this.getParent().eliminarComponente(posicion);
			
			// Insertar en la superior
			cc.agregarComponente(this, cc.getComponentes().size());
			
		}
		
		return 3;

	}
	/**
	 * Desplazar hacia izquierda (nivel de anidamiento inferior) la componente, en el caso de que sto sea posible.
	 */
	public void desplazarHaciaIzquierda() {
		
		int posicion = parent.getPosicionComponente(this.getId());
		
		// Quitar la componente actual del parent
		
		parent.eliminarComponente(posicion);
		
		// Recuperar la posicin del parent en su propio parent
		
		int posicionParent = parent.getParent().getPosicionComponente(parent.getId());
		
		// Aadir la componente actual a continuacin de su parent anterior
		
		parent.getParent().getComponentes().add(posicionParent+1, this);
		
		// Modificar el parent de la componente actual
		
		parent = parent.getParent();
		
		// Actualizar su nivel de anidamiento
		
		setAnidamiento(parent.getAnidamiento()+1);
		
		// Actualizar los smbolos del parent
		
		parent.actualizarSimbolos();
		
	}
	
	/**
	 * Genera un array de PropiedadesLinea, a partir del prototipo PrototipoListaPropiedadesLinea.
	 *
	 * @return array con la lista de objetos PropiedadLinea asociados a la componente.
	 */
	public ArrayList<PropiedadLinea> generarPropiedades(){
		PrototipoListaPropiedadLinea plpl = PrototipoListaPropiedadLinea.getInstance();
		ArrayList<PropiedadLinea> lista =  (ArrayList<PropiedadLinea>) plpl.clone();
		
		for(PropiedadLinea p: lista)
			p.setComponente(this);
		
		return lista;
	}
	
	/**
	 * La componente pasada como parmetro se establece como derivada y se actualizan sus campos
	 * ComponenteOrigen y EstructuraOrigen, estableciendo su valor al identificador de la componente
	 * actual.
	 *
	 * @param componente Componente Derivada
	 */
	public void generarReferenciasDerivacion(Componente componente){
		componente.setDerivada(true);
		componente.setIdComponenteOrigen(getId());
		componente.setIdEstructuraOrigen(getEstructura().getId());
		componente.setId(UUID.randomUUID().toString());
	}
	
	/**
	 * Recupera anidamiento.
	 *
	 * @return el anidamiento
	 */
	public int getAnidamiento() {
		return anidamiento;
	}
	
	/**
	 * Recupera derivada.
	 *
	 * @return el derivada
	 */
	public Boolean getDerivada() {
		return derivada;
	}
	
	/**
	 * Recupera estructura.
	 *
	 * @return el estructura
	 */
	public Estructura getEstructura() {
		return estructura;
	}



	/**
	 * Recupera id.
	 *
	 * @return el id
	 */
	public String getId() {
		return id;
	}



	/**
	 * Recupera id componente origen.
	 *
	 * @return el id componente origen
	 */
	public String getIdComponenteOrigen() {
		return idComponenteOrigen;
	}
	
	
	
	/**
	 * Recupera id estructura origen.
	 *
	 * @return el id estructura origen
	 */
	public String getIdEstructuraOrigen() {
		return idEstructuraOrigen;
	}



	/**
	 * Obtiene todas las instancias de componenteLinea de la componente actual, 
	 * as como las de las subcomponente mediante la propagacin de este mtodo 
	 * a las subcomponentes de las componentes complejas.
	 * 
	 * De esta forma se construye el array AALC de las estructuras.
	 *
	 * @return array con las lneas ordenadas de la componente y sus subcomponentes
	 */
	public abstract ArrayList<Linea> getLineas();



	/**
	 * Recupera lineas sin simbolos.
	 *
	 * @return el lineas sin simbolos
	 */
	public abstract ArrayList<ComponenteLinea> getLineasSinSimbolos();

	/**
	 * Recupera nombre.
	 *
	 * @return el nombre
	 */
	public String getNombre() {
		return nombre;
	}
	
	/**
	 * Recupera opcional.
	 *
	 * @return el opcional
	 */
	public Boolean getOpcional() {
		return opcional;
	}

	/**
	 * Recupera parent.
	 *
	 * @return el parent
	 */
	public ComponenteCompuesta getParent() {
		return parent;
	}



	/**
	 * Recupera propiedades.
	 *
	 * @return el propiedades
	 */
	public ArrayList<PropiedadLinea> getPropiedades() {
		return propiedades;
	}



	/**
	 * Insertar en diccionario.
	 */
	public abstract void insertarEnDiccionario();

	/**
	 * Modicar opcionalidad.
	 */
	public abstract void modicarOpcionalidad();
	
	/**
	 * Modificar clase.
	 */
	public abstract void modificarClase();
	
	/**
	 * Obtener nicamente una parte del arbol que forman la ruta hacia una estructura o componente
	 *
	 * @return objeto DefaultMutableTreeNode que contiene la ruta
	 */
	public abstract DefaultMutableTreeNode obtenerSubArbol();



	/**
	 * Recuperacin de los punteros hacia atrs de las componentes, las lneasComponente y las lneasPropiedad.
	 * Estas referencias haban sido perdidas al realizar la serializacin y deserializacin del objeto en XML.
	 */
	public abstract void recuperarReferencias();



	/**
	 * Establece anidamiento.
	 *
	 * @param anidamiento el nuevo anidamiento
	 */
	public void setAnidamiento(int anidamiento) {
		this.anidamiento = anidamiento;
	}



	/**
	 * Establece derivada.
	 *
	 * @param derivada el nuevo derivada
	 */
	public void setDerivada(Boolean derivada) {
		this.derivada = derivada;
	}



	/**
	 * Establece estructura.
	 *
	 * @param estructura el nuevo estructura
	 */
	public abstract void setEstructura(Estructura estructura);



	/**
	 * Establece id.
	 *
	 * @param id el nuevo id
	 */
	public void setId(String id) {
		this.id = id;
	}



	/**
	 * Establece id componente origen.
	 *
	 * @param idComponenteOrigen el nuevo id componente origen
	 */
	public void setIdComponenteOrigen(String idComponenteOrigen) {
		this.idComponenteOrigen = idComponenteOrigen;
	}

	/**
	 * Establece id estructura origen.
	 *
	 * @param idEstructuraOrigen el nuevo id estructura origen
	 */
	public void setIdEstructuraOrigen(String idEstructuraOrigen) {
		this.idEstructuraOrigen = idEstructuraOrigen;
	}
	
	/**
	 * Establece nombre.
	 *
	 * @param nombre el nuevo nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	
	/**
	 * Establece opcional.
	 *
	 * @param opcional el nuevo opcional
	 */
	public void setOpcional(Boolean opcional) {
		this.opcional = opcional;
	}
	
	/**
	 * Establece parent.
	 *
	 * @param parent el nuevo parent
	 */
	public void setParent(ComponenteCompuesta parent) {
		this.parent = parent;
	}
	
	/**
	 * Establece propiedades.
	 *
	 * @param propiedades el nuevo propiedades
	 */
	public void setPropiedades(ArrayList<PropiedadLinea> propiedades) {
		this.propiedades = propiedades;
	}
	
	/**
	 * Validacin de una componente. Probar que los valores de las propiedades introducidos se ajustan al tipo de datos
	 * esperado. Para las componetnes complejas se propaga la llamada a las subcomponentes.
	 *
	 * @param presentacion Presentacin abierta en el editor
	 * @return informe de errores localizados
	 */
	public abstract String validar(Presentacion presentacion);



	@Override
	public String toString() {
		return this.getNombre();
	}
	
	
}
