package modelo;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.UUID;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;

import modelo.prototypes.PrototipoListaAcabado;
import modelo.prototypes.PrototipoListaAspecto;
import modelo.prototypes.PrototipoListaPropiedades;

// TODO: Auto-generated Javadoc
/**
 * Una presentacin define las propiedades visibles en un momento dado en el editor, as como el tipo de valores que 
 * aceptan dichas propiedades. Adems, contiene los valores de las caractersticas de acabado y aspecto que complementan
 * la descripcin grfica de las componentes mostradas en el editor.
 */
@XmlAccessorType(XmlAccessType.NONE)
@XmlRootElement(name="Presentacion")
public class Presentacion {
	
	/** Constante PRIVADA. */
	public static final int PRIVADA = 0;
	
	/** Constante PUBLICA. */
	public static final int PUBLICA = 1;
	
	/** Constante DEFAULT. */
	public static final int DEFAULT = 2;
	
	/** Activacin del modo depuracin. */
	private static final boolean DEBUG = false;
	
	/** Identificador. */
	@XmlID
	private String id = UUID.randomUUID().toString();
	
	/** Nombre de la presentacin. */
	@XmlAttribute(name="Nombre")
	private String nombre;
	
	/** Tipo de la presentacin (Presentacion.PRIVADA/PUBLICA/DEFAULT). */
	@XmlAttribute(name="Tipo")
	private int tipo;
	
	/** Fecha de creacin de la presentacin. */
	@XmlAttribute(name="FechaCreacion")
	private Calendar creacion;

	/** Fecha de la ltima modificacin de la presentacin. */
	@XmlAttribute(name="FechaModificacion")
	private Calendar modificacion;
	
	/** Conjunto de propiedades que contiene la presentacin. */
	@XmlElement(name="Propiedad")
	private ArrayList<Propiedad> propiedades;
	
	/** Conjunto de caractersticas de aspecto. */
	@XmlElement(name="CaracteristicaAspecto")
	private ArrayList<Aspecto> aspectos;
	
	/** Conjunto de caractersiticas de acabado. */
	@XmlElement(name="CaracteristicaAcabado")
	private ArrayList<Acabado> acabados;
	
	/**
	 * Inicializacin de una nueva presentacin.
	 */
	public Presentacion() {
	}

	/**
	 * Inicializa un objeto de tipo presentacion.
	 *
	 * @param nombre nombre de la presentacin
	 * @param tipo tipo de la presentacin
	 */
	public Presentacion(String nombre, int tipo) {
		super();
		this.nombre = nombre;
		this.tipo = tipo;
		this.creacion = Calendar.getInstance();
		this.modificacion = Calendar.getInstance();
		
		// La lista de propiedades se copia de un prototipo (patrn prototype) que obtiene
		// la lista de propiedades de fichero. Si el fichero no exisite o est corrupto,
		// lo vuelve a generar.
		PrototipoListaPropiedades plp = new PrototipoListaPropiedades();
		plp.abrir();
		
		this.propiedades = (ArrayList<Propiedad>) plp.clone();
		
		PrototipoListaAspecto pla = PrototipoListaAspecto.getInstance();
		this.aspectos = (ArrayList<Aspecto>) pla.clone();
		
		PrototipoListaAcabado plac = PrototipoListaAcabado.getInstance();
		this.acabados = (ArrayList<Acabado>) plac.clone();
		
		// Actualizar referencias de las propiedades clonadas
		
		for(Propiedad p : propiedades){
			p.setPresentacion(this);
		}
	}

	/**
	 * Recupera creacion.
	 *
	 * @return la fecha de creacin
	 */
	public Calendar getCreacion() {
		return creacion;
	}
	
	
	
	/**
	 * Recupera id.
	 *
	 * @return el id
	 */
	public String getId() {
		return id;
	}

	/**
	 * Recupera la fecha de modificacin.
	 *
	 * @return la fecha de modificacin
	 */
	public Calendar getModificacion() {
		return modificacion;
	}
	
	/**
	 * Recupera el nombre.
	 *
	 * @return el nombre
	 */
	public String getNombre() {
		return nombre;
	}

	/**
	 * Gets the propiedades.
	 *
	 * @return the propiedades
	 */
	public ArrayList<Propiedad> getPropiedades() {
		return propiedades;
	}

	/**
	 * Recupera tipo.
	 *
	 * @return el tipo
	 */
	public int getTipo() {
		return tipo;
	}

	/**
	 * Marca todas las propiedades como visibles.
	 */
	public void mostrarTodasPropiedades(){
		for(int i = 0; i < propiedades.size(); i++)
			propiedades.get(i).setVisible(true);
	}

	/**
	 * Establece creacion.
	 *
	 * @param creacion el nuevo creacion
	 */
	public void setCreacion(Calendar creacion) {
		this.creacion = creacion;
	}
	
	

	/**
	 * Establece id.
	 *
	 * @param id el nuevo id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Establece modificacion.
	 *
	 * @param modificacion el nuevo modificacion
	 */
	public void setModificacion(Calendar modificacion) {
		this.modificacion = modificacion;
	}

	/**
	 * Establece nombre.
	 *
	 * @param nombre el nuevo nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	/**
	 * Sets the propiedades.
	 *
	 * @param propiedades the new propiedades
	 */
	public void setPropiedades(ArrayList<Propiedad> propiedades) {
		this.propiedades = propiedades;
	}

	/**
	 * Establece tipo.
	 *
	 * @param tipo el nuevo tipo
	 */
	public void setTipo(int tipo) {
		this.tipo = tipo;
	}

	/**
	 * Recupera aspectos.
	 *
	 * @return el aspectos
	 */
	public ArrayList<Aspecto> getAspectos() {
		return aspectos;
	}

	/**
	 * Establece aspectos.
	 *
	 * @param aspectos el nuevo aspectos
	 */
	public void setAspectos(ArrayList<Aspecto> aspectos) {
		this.aspectos = aspectos;
	}

	/**
	 * Recupera acabados.
	 *
	 * @return el acabados
	 */
	public ArrayList<Acabado> getAcabados() {
		return acabados;
	}

	/**
	 * Establece acabados.
	 *
	 * @param acabados el nuevo acabados
	 */
	public void setAcabados(ArrayList<Acabado> acabados) {
		this.acabados = acabados;
	}

	@Override
	public String toString() {
		return this.getNombre();
	}
	
	
	
}
