package modelo;

import java.util.UUID;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlSeeAlso;

import modelo.tipoDatos.Alfanumerico;
import modelo.tipoDatos.Binario;
import modelo.tipoDatos.Numerico;
import modelo.tipoDatos.Rango;

// TODO: Auto-generated Javadoc
/**
 * Tipo de datos de una propiedad. Genera una expresin regular que posteriormente es comprobada mediante
 * la clase Pattern de java.
 */
@XmlAccessorType(XmlAccessType.NONE)
@XmlSeeAlso({Alfanumerico.class, Numerico.class, Rango.class, Binario.class})
public class TipoDatos implements Cloneable{
	
	/** Identificador. */
	@XmlID
	private String id = UUID.randomUUID().toString();
	
	/** Es nulo?. */
	@XmlAttribute(name="Nulo")
	private boolean nulo;
	
	/** Es alfanumrico?. */
	@XmlAttribute(name="Alfanumerico")
	private boolean alfanumerico;
	
	/** Es numrico?. */
	@XmlAttribute(name="Numerico")
	private boolean numerico;
	
	/** Es rango?. */
	@XmlAttribute(name="Rango")
	private boolean rango;
	
	/** Es binario?. */
	@XmlAttribute(name="Binario")
	private boolean binario;
	
	/** Es negativo?. */
	@XmlAttribute(name="NoNegativo")
	private boolean noNegativo;
	
	/** Longitud mxima. */
	@XmlAttribute(name="LongitudMaxima")
	private int longMax;
	
	/** Longitud mnima. */
	@XmlAttribute(name="LongitudMinima")
	private int longMin;
	
	/** Expresin regular formada a partir de los atributos restantes. */
	@XmlAttribute(name="ExpresionRegular")
	private String expresionRegular;
	
	/** Descripcin de la expresin regular. Usada a la hora de la generacin de informes de errores al validar
	 * una estructura */
	@XmlAttribute(name="Descripcion")
	private String descripcion;
	
	/**
	 * Inicializa un objeto de tipo tipo datos.
	 */
	public TipoDatos() {
		this.nulo = true;
		this.alfanumerico = false;
		this.numerico = false;
		this.rango = false;
		this.binario = false;
		this.noNegativo = false;
		this.longMax = 0;
		this.longMin = 0;
		this.expresionRegular = "";
		this.descripcion = "";
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {
		TipoDatos clonado = new TipoDatos();
		clonado.setAlfanumerico(alfanumerico);
		clonado.setBinario(binario);
		clonado.setId(id);
		clonado.setLongMax(longMax);
		clonado.setLongMin(longMin);
		clonado.setNoNegativo(noNegativo);
		clonado.setNulo(nulo);
		clonado.setNumerico(numerico);
		clonado.setRango(rango);
		clonado.generarExpresion();
		return clonado;
	}
	
	/**
	 * Generar la descripcin del tipo de datos esperado a partir de los otros atributos.
	 */
	protected void generarDescripcion() {
		StringBuffer sb = new StringBuffer();
		if(isAlfanumerico()){
			sb.append("alfanumrico de longitud entre " + longMin + " y " + longMax + ".");
		}
		else if(isNumerico()){
			if(noNegativo){
				sb.append("numrico no negativo");
			}
			else{
				sb.append("numrico");
			}
		}
		else if(isRango()){
			sb.append("rango de valores (p.e. [1,5])");
		}
		else if(isBinario()){
			sb.append("binario (\"si\" o \"no\")");
		}
		descripcion = sb.toString();
		
	}
	
	/**
	 * Generar la expresin regular del tipo de datos esperado a partir de los otros atributos.
	 */
	public void generarExpresion(){
		if(isAlfanumerico()){
			expresionRegular = "(.{" + longMin + "," + longMax + "})?";
		}
		else if(isNumerico()){
			if(noNegativo){
				expresionRegular = "(([0-9]+(\\.[0-9]+)?))?";
			}
			else{
				
				expresionRegular = "((-)?([0-9]+(\\.[0-9]+)?))?";
			}
		}
		else if(isRango()){
			String numero = "([0-9]+(\\.[0-9]+)?)";
			expresionRegular = "(\\[" + numero + "," + numero + "\\])?";
		}
		else if(isBinario()){
			expresionRegular = "(si|no)?" ;
		}
		generarDescripcion();
	}
	
	/**
	 * Recupera descripcion.
	 *
	 * @return el descripcion
	 */
	public String getDescripcion() {
		return descripcion;
	}
	
	/**
	 * Recupera expresion regular.
	 *
	 * @return el expresion regular
	 */
	public String getExpresionRegular() {
		return expresionRegular;
	}
	
	/**
	 * Recupera id.
	 *
	 * @return el id
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * Recupera long max.
	 *
	 * @return el long max
	 */
	public int getLongMax() {
		return longMax;
	}
	
	/**
	 * Recupera long min.
	 *
	 * @return el long min
	 */
	public int getLongMin() {
		return longMin;
	}
	
	/**
	 * Comprueba si alfanumerico.
	 *
	 * @return true, if is alfanumerico
	 */
	public boolean isAlfanumerico() {
		return alfanumerico;
	}
	
	/**
	 * Comprueba si binario.
	 *
	 * @return true, if is binario
	 */
	public boolean isBinario() {
		return binario;
	}
	
	/**
	 * Comprueba si no negativo.
	 *
	 * @return true, if is no negativo
	 */
	public boolean isNoNegativo() {
		return noNegativo;
	}
	
	/**
	 * Comprueba si nulo.
	 *
	 * @return true, if is nulo
	 */
	public boolean isNulo() {
		return nulo;
	}
	
	/**
	 * Comprueba si numerico.
	 *
	 * @return true, if is numerico
	 */
	public boolean isNumerico() {
		return numerico;
	}
	
	/**
	 * Comprueba si rango.
	 *
	 * @return true, if is rango
	 */
	public boolean isRango() {
		return rango;
	}
	
	/**
	 * Reset.
	 */
	public void reset(){
		alfanumerico = false;
		numerico = false;
		binario = false;
		rango = false;
	}
	
	/**
	 * Establece alfanumerico.
	 *
	 * @param alfanumerico el nuevo alfanumerico
	 */
	public void setAlfanumerico(boolean alfanumerico) {
		this.alfanumerico = alfanumerico;
	}
	
	/**
	 * Establece binario.
	 *
	 * @param binario el nuevo binario
	 */
	public void setBinario(boolean binario) {
		this.binario = binario;
	}
	
	/**
	 * Establece descripcion.
	 *
	 * @param descripcion el nuevo descripcion
	 */
	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}
	
	/**
	 * Establece expresion regular.
	 *
	 * @param expresionRegular el nuevo expresion regular
	 */
	public void setExpresionRegular(String expresionRegular) {
		this.expresionRegular = expresionRegular;
	}
	
	/**
	 * Establece id.
	 *
	 * @param id el nuevo id
	 */
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * Establece long max.
	 *
	 * @param longMax el nuevo long max
	 */
	public void setLongMax(int longMax) {
		this.longMax = longMax;
	}
	
	/**
	 * Establece long min.
	 *
	 * @param longMin el nuevo long min
	 */
	public void setLongMin(int longMin) {
		this.longMin = longMin;
	}
	
	/**
	 * Establece no negativo.
	 *
	 * @param noNegativo el nuevo no negativo
	 */
	public void setNoNegativo(boolean noNegativo) {
		this.noNegativo = noNegativo;
	}
	
	/**
	 * Establece nulo.
	 *
	 * @param nulo el nuevo nulo
	 */
	public void setNulo(boolean nulo) {
		this.nulo = nulo;
		expresionRegular = "";
	}
	
	/**
	 * Establece numerico.
	 *
	 * @param numerico el nuevo numerico
	 */
	public void setNumerico(boolean numerico) {
		this.numerico = numerico;
	}
	
	/**
	 * Establece rango.
	 *
	 * @param rango el nuevo rango
	 */
	public void setRango(boolean rango) {
		this.rango = rango;
	}
	
	
	
}
