package utils;

import java.util.ArrayList;

import datos.PresentacionesDAO;

import modelo.Presentacion;

// TODO: Auto-generated Javadoc
/**
 * ManagerPresentaciones se encarga de la gestin de las presentaciones de la aplicacin.
 * Contiene un listado de las privadas, publicas y default.
 * Realiza llamadas al DAO cada vez que se crea/elimina/modifica una presentacin.
 */
public class ManagerPresentaciones {
    
    /** INSTANCE. */
    private static ManagerPresentaciones INSTANCE = null;
    
    /** Conjunto de presentaciones pblicas. */
    private ArrayList<Presentacion> presentacionesPublicas = null;
    
    /** Conjunto de presentaciones privadas. */
    private ArrayList<Presentacion> presentacionesPrivadas = null;
    
    /** Presentacin default */
    private Presentacion presentacionDefault = null;
    
    /** Data access object. */
    private PresentacionesDAO dao = PresentacionesDAO.getInstance();
    
    // Private constructor suppresses 
    /**
     * Inicializa un objeto de tipo manager presentaciones.
     */
    private ManagerPresentaciones() {
    	inicializar();
    }
 
    /**
     * Inicializar el manager. Carga las presentaciones existentes de disco, y en el caso de que no exista ninguna,
     * crea una presentacin default.
     */
    private void inicializar() {
    	PresentacionesDAO dao = PresentacionesDAO.getInstance();
		ArrayList<Presentacion> presentaciones = dao.cargarPresentaciones();
		presentacionesPublicas = new ArrayList<Presentacion>();
		presentacionesPrivadas = new ArrayList<Presentacion>();
		
		for(Presentacion p: presentaciones){
			switch(p.getTipo()){
			case Presentacion.PRIVADA:
				presentacionesPrivadas.add(p);
				break;
			case Presentacion.PUBLICA:
				presentacionesPublicas.add(p);
				break;
			case Presentacion.DEFAULT:
				presentacionesPublicas.add(p);
				presentacionDefault = p;
				break;
			}
		}
		
		if(presentacionDefault == null)
			this.crearPresentacionDefault();
		
	}

	/**
	 * Crear presentacion default.
	 */
	private void crearPresentacionDefault() {
		presentacionDefault = new Presentacion("Default1", Presentacion.DEFAULT);
		presentacionesPublicas.add(presentacionDefault);
	}

	// creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciacin mltiple 
    /**
	 * Crea instance.
	 */
	private synchronized static void createInstance() {
        if (INSTANCE == null) { 
            INSTANCE = new ManagerPresentaciones();
        }
    }
    
    /**
     * Busca una presentacin a travs de su nombre
     *
     * @param nombre nombre de la presentacin
     * @return la presentacin encontrada, o null en caso de que no se encuentre la presentacin buscada.
     */
    public Presentacion buscarPorId(String id){
    	for(Presentacion p: presentacionesPublicas){
    		if(p.getId().equals(id)) return p;
    	}
    	for(Presentacion p: presentacionesPrivadas){
    		if(p.getId().equals(id)) return p;
    	}
    	return null;
    }
    
    public Presentacion buscarPorNombre(String nombre){
    	for(Presentacion p: presentacionesPublicas){
    		if(p.getNombre().equals(nombre)) return p;
    	}
    	for(Presentacion p: presentacionesPrivadas){
    		if(p.getNombre().equals(nombre)) return p;
    	}
    	return null;
    }
 
    /**
     * Gets the single instance of ManagerPresentaciones.
     *
     * @return single instance of ManagerPresentaciones
     */
    public static ManagerPresentaciones getInstance() {
        createInstance();
        return INSTANCE;
    }

	/**
	 * Recupera presentaciones publicas.
	 *
	 * @return el presentaciones publicas
	 */
	public ArrayList<Presentacion> getPresentacionesPublicas() {
		return presentacionesPublicas;
	}
//
//	public void setPresentacionesPublicas(
//			ArrayList<Presentacion> presentacionesPublicas) {
//		this.presentacionesPublicas = presentacionesPublicas;
//	}
//
	/**
 * Recupera presentaciones privadas.
 *
 * @return el presentaciones privadas
 */
public ArrayList<Presentacion> getPresentacionesPrivadas() {
		return presentacionesPrivadas;
	}
//
//	public void setPresentacionesPrivadas(
//			ArrayList<Presentacion> presentacionesPrivadas) {
//		this.presentacionesPrivadas = presentacionesPrivadas;
//	}
//
	/**
 * Recupera presentacion default.
 *
 * @return el presentacion default
 */
public Presentacion getPresentacionDefault() {
		return presentacionDefault;
	}
//
//	public void setPresentacionDefault(Presentacion presentacionDefault) {
//		this.presentacionDefault = presentacionDefault;
//	}

    /**
 * Modificar presentacion.
 *
 * @param p presentacin a modificar
 */
public void modificarPresentacion(Presentacion p){
    	dao.eliminarPresentacion(p.getNombre());
    	dao.guardarPresentacin(p);
    	return;
    }

	/**
	 * Agregar presentacion publica.
	 *
	 * @param p La nueva presentacin pblica.
	 */
	public void agregarPresentacionPublica(Presentacion p){
		this.presentacionesPublicas.add(p);
		dao.guardarPresentacin(p);
	}
	
	/**
	 * Agregar presentacion privada.
	 *
	 * @param p La nueva presentacin privada.
	 */
	public void agregarPresentacionPrivada(Presentacion p){
		this.presentacionesPrivadas.add(p);
		dao.guardarPresentacin(p);
	}
	
	/**
	 * Agregar presentacion default.
	 *
	 * @param p La nueva presentacin default.
	 */
	public void agregarPresentacionDefault(Presentacion p){
		this.presentacionDefault = p;
		this.presentacionesPublicas.add(p);
		dao.guardarPresentacin(p);
	}
	
	/**
	 * Eliminar presentacion publica.
	 *
	 * @param p La presentacin pblica a eliminar.
	 */
	public void eliminarPresentacionPublica(Presentacion p){
		this.presentacionesPublicas.remove(p);
		dao.eliminarPresentacion(p.getNombre());
	}
	
	/**
	 * Eliminar presentacion privada.
	 *
	 * @param p La presentacin privada a eliminar.
	 */
	public void eliminarPresentacionPrivada(Presentacion p){
		this.presentacionesPrivadas.remove(p);
		dao.eliminarPresentacion(p.getNombre());
	}
	
	/**
	 * Eliminar presentacion default.
	 *
	 * @param p La presentacin default a eliminar.
	 */
	public void eliminarPresentacionDefault(Presentacion p){
		this.presentacionesPublicas.remove(p);
		this.presentacionDefault = null;
		dao.eliminarPresentacion(p.getNombre());
	}

	/**
	 * Modificar presentacion.
	 *
	 * @param presentacion la presentacin a modificar.
	 * @param nombreAntiguo el antiguo nombre de la presentacin, antes de realiar empezar a modificarla.
	 */
	public void modificarPresentacion(Presentacion presentacion,
			String nombreAntiguo) {
    	dao.eliminarPresentacion(nombreAntiguo);
    	dao.guardarPresentacin(presentacion);
    	return;
		
	}
	
	public ArrayList<String> listarNombrePresentaciones(){
		ArrayList<String> nombres = new ArrayList<String>();
		
		for(Presentacion p: this.getPresentacionesPrivadas()){
			nombres.add(p.getNombre());
		}
		
		for(Presentacion p: this.getPresentacionesPublicas()){
			nombres.add(p.getNombre());
		}
		
		return nombres;
	}
    
    
}